"""init

Revision ID: 1bdba1540905
Revises: 
Create Date: 2023-07-24 11:26:30.661710+01:00

"""
import sqlalchemy as sa
import sqlmodel
from alembic import op

# revision identifiers, used by Alembic.
revision = "1bdba1540905"
down_revision = None
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "active_project",
        sa.Column("project_ontology", sa.JSON(), nullable=True),
        sa.Column("custom_metrics", sa.JSON(), nullable=True),
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("project_name", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("project_description", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("project_remote_ssh_key_path", sqlmodel.sql.sqltypes.AutoString(), nullable=True),
        sa.PrimaryKeyConstraint("project_hash"),
    )
    op.create_table(
        "active_project_data",
        sa.Column("label_row_json", sa.JSON(), nullable=True),
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("data_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("label_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("dataset_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("num_frames", sa.Integer(), nullable=False),
        sa.Column("frames_per_second", sa.Float(), nullable=True),
        sa.Column("dataset_title", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("data_title", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("data_type", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.ForeignKeyConstraint(
            ["project_hash"],
            ["active_project.project_hash"],
            name="active_project_data_project_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("project_hash", "data_hash"),
        sa.UniqueConstraint("label_hash"),
    )
    op.create_table(
        "active_project_embedding_reduction",
        sa.Column("reduction_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("reduction_name", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("reduction_description", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("reduction_type", sa.Enum("UMAP", name="embeddingreductiontype"), nullable=False),
        sa.Column("reduction_bytes", sa.LargeBinary(), nullable=False),
        sa.ForeignKeyConstraint(
            ["project_hash"],
            ["active_project.project_hash"],
            name="active_project_embedding_reduction_project_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("reduction_hash"),
    )
    op.create_table(
        "active_project_prediction",
        sa.Column("prediction_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("name", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.ForeignKeyConstraint(
            ["project_hash"],
            ["active_project.project_hash"],
            name="active_project_prediction_project_hash_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("prediction_hash"),
    )
    op.create_table(
        "active_project_tags",
        sa.Column("tag_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("name", sqlmodel.sql.sqltypes.AutoString(length=256), nullable=False),
        sa.Column("description", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.ForeignKeyConstraint(
            ["project_hash"],
            ["active_project.project_hash"],
            name="active_project_tags_project_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("tag_hash"),
    )
    op.create_index(op.f("ix_active_project_tags_project_hash"), "active_project_tags", ["project_hash"], unique=False)
    op.create_table(
        "active_project_data_units",
        sa.Column("objects", sa.JSON(), nullable=True),
        sa.Column("classifications", sa.JSON(), nullable=True),
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("data_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("width", sa.Integer(), nullable=False),
        sa.Column("height", sa.Integer(), nullable=False),
        sa.Column("data_uri", sqlmodel.sql.sqltypes.AutoString(), nullable=True),
        sa.Column("data_uri_is_video", sa.Boolean(), nullable=False),
        sa.ForeignKeyConstraint(
            ["project_hash", "data_hash"],
            ["active_project_data.project_hash", "active_project_data.data_hash"],
            name="active_data_unit_data_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.ForeignKeyConstraint(
            ["project_hash"],
            ["active_project.project_hash"],
        ),
        sa.PrimaryKeyConstraint("project_hash", "du_hash", "frame"),
    )
    op.create_index(
        "active_project_data_units_unique_du_hash_frame",
        "active_project_data_units",
        ["project_hash", "du_hash", "frame"],
        unique=True,
    )
    op.create_table(
        "active_project_prediction_analytics",
        sa.Column(
            "annotation_type",
            sa.Enum(
                "CLASSIFICATION",
                "BOUNDING_BOX",
                "ROT_BOUNDING_BOX",
                "POINT",
                "POLYLINE",
                "POLYGON",
                "SKELETON",
                "BITMASK",
                name="annotationtype",
            ),
            nullable=True,
        ),
        sa.Column("prediction_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("object_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=False),
        sa.Column("feature_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=False),
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("match_object_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=True),
        sa.Column("match_feature_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=True),
        sa.Column("match_duplicate_iou", sa.Float(), nullable=False),
        sa.Column("iou", sa.Float(), nullable=False),
        sa.Column("metric_width", sa.Integer(), nullable=True),
        sa.Column("metric_height", sa.Integer(), nullable=True),
        sa.Column("metric_area", sa.Integer(), nullable=True),
        sa.Column("metric_area_relative", sa.Float(), nullable=True),
        sa.Column("metric_aspect_ratio", sa.Float(), nullable=True),
        sa.Column("metric_brightness", sa.Float(), nullable=True),
        sa.Column("metric_contrast", sa.Float(), nullable=True),
        sa.Column("metric_sharpness", sa.Float(), nullable=True),
        sa.Column("metric_red", sa.Float(), nullable=True),
        sa.Column("metric_green", sa.Float(), nullable=True),
        sa.Column("metric_blue", sa.Float(), nullable=True),
        sa.Column("metric_random", sa.Float(), nullable=True),
        sa.Column("metric_annotation_quality", sa.Float(), nullable=True),
        sa.Column("metric_label_duplicates", sa.Float(), nullable=True),
        sa.Column("metric_label_border_closeness", sa.Float(), nullable=True),
        sa.Column("metric_label_poly_similarity", sa.Float(), nullable=True),
        sa.Column("metric_label_missing_or_broken_tracks", sa.Float(), nullable=True),
        sa.Column("metric_label_inconsistent_classification_and_track", sa.Float(), nullable=True),
        sa.Column("metric_label_shape_outlier", sa.Float(), nullable=True),
        sa.Column("metric_label_confidence", sa.Float(), nullable=False),
        sa.Column("metric_custom0", sa.Float(), nullable=True),
        sa.Column("metric_custom1", sa.Float(), nullable=True),
        sa.Column("metric_custom2", sa.Float(), nullable=True),
        sa.Column("metric_custom3", sa.Float(), nullable=True),
        sa.ForeignKeyConstraint(
            ["prediction_hash"],
            ["active_project_prediction.prediction_hash"],
            name="active_project_prediction_objects_prediction_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("prediction_hash", "du_hash", "frame", "object_hash"),
    )
    op.create_index(
        "active_project_prediction_objects_confidence_index",
        "active_project_prediction_analytics",
        ["prediction_hash", "metric_label_confidence"],
        unique=False,
    )
    op.create_index(
        "active_project_prediction_objects_feature_confidence_index",
        "active_project_prediction_analytics",
        ["prediction_hash", "feature_hash", "metric_label_confidence"],
        unique=False,
    )
    op.create_table(
        "active_project_prediction_analytics_false_negatives",
        sa.Column(
            "annotation_type",
            sa.Enum(
                "CLASSIFICATION",
                "BOUNDING_BOX",
                "ROT_BOUNDING_BOX",
                "POINT",
                "POLYLINE",
                "POLYGON",
                "SKELETON",
                "BITMASK",
                name="annotationtype",
            ),
            nullable=True,
        ),
        sa.Column("prediction_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("object_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=False),
        sa.Column("iou_threshold", sa.Float(), nullable=False),
        sa.Column("feature_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=False),
        sa.Column("annotation_email", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("annotation_manual", sa.Boolean(), nullable=False),
        sa.Column("metric_width", sa.Integer(), nullable=True),
        sa.Column("metric_height", sa.Integer(), nullable=True),
        sa.Column("metric_area", sa.Integer(), nullable=True),
        sa.Column("metric_area_relative", sa.Float(), nullable=True),
        sa.Column("metric_aspect_ratio", sa.Float(), nullable=True),
        sa.Column("metric_brightness", sa.Float(), nullable=True),
        sa.Column("metric_contrast", sa.Float(), nullable=True),
        sa.Column("metric_sharpness", sa.Float(), nullable=True),
        sa.Column("metric_red", sa.Float(), nullable=True),
        sa.Column("metric_green", sa.Float(), nullable=True),
        sa.Column("metric_blue", sa.Float(), nullable=True),
        sa.Column("metric_random", sa.Float(), nullable=True),
        sa.Column("metric_annotation_quality", sa.Float(), nullable=True),
        sa.Column("metric_label_duplicates", sa.Float(), nullable=True),
        sa.Column("metric_label_border_closeness", sa.Float(), nullable=True),
        sa.Column("metric_label_poly_similarity", sa.Float(), nullable=True),
        sa.Column("metric_label_missing_or_broken_tracks", sa.Float(), nullable=True),
        sa.Column("metric_label_inconsistent_classification_and_track", sa.Float(), nullable=True),
        sa.Column("metric_label_shape_outlier", sa.Float(), nullable=True),
        sa.Column("metric_label_confidence", sa.Float(), nullable=False),
        sa.Column("metric_custom0", sa.Float(), nullable=True),
        sa.Column("metric_custom1", sa.Float(), nullable=True),
        sa.Column("metric_custom2", sa.Float(), nullable=True),
        sa.Column("metric_custom3", sa.Float(), nullable=True),
        sa.ForeignKeyConstraint(
            ["prediction_hash"],
            ["active_project_prediction.prediction_hash"],
            name="active_project_prediction_unmatched_prediction_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("prediction_hash", "du_hash", "frame", "object_hash"),
    )
    op.create_index(
        "active_project_prediction_unmatched_feature_hash_index",
        "active_project_prediction_analytics_false_negatives",
        ["prediction_hash", "feature_hash"],
        unique=False,
    )
    op.create_table(
        "active_project_analytics_annotation",
        sa.Column(
            "annotation_type",
            sa.Enum(
                "CLASSIFICATION",
                "BOUNDING_BOX",
                "ROT_BOUNDING_BOX",
                "POINT",
                "POLYLINE",
                "POLYGON",
                "SKELETON",
                "BITMASK",
                name="annotationtype",
            ),
            nullable=True,
        ),
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("object_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=False),
        sa.Column("feature_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=False),
        sa.Column("annotation_email", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("annotation_manual", sa.Boolean(), nullable=False),
        sa.Column("embedding_clip", sa.LargeBinary(), nullable=True),
        sa.Column("embedding_hu", sa.LargeBinary(), nullable=True),
        sa.Column("metric_width", sa.Integer(), nullable=True),
        sa.Column("metric_height", sa.Integer(), nullable=True),
        sa.Column("metric_area", sa.Integer(), nullable=True),
        sa.Column("metric_area_relative", sa.Float(), nullable=True),
        sa.Column("metric_aspect_ratio", sa.Float(), nullable=True),
        sa.Column("metric_brightness", sa.Float(), nullable=True),
        sa.Column("metric_contrast", sa.Float(), nullable=True),
        sa.Column("metric_sharpness", sa.Float(), nullable=True),
        sa.Column("metric_red", sa.Float(), nullable=True),
        sa.Column("metric_green", sa.Float(), nullable=True),
        sa.Column("metric_blue", sa.Float(), nullable=True),
        sa.Column("metric_random", sa.Float(), nullable=True),
        sa.Column("metric_annotation_quality", sa.Float(), nullable=True),
        sa.Column("metric_label_duplicates", sa.Float(), nullable=True),
        sa.Column("metric_label_border_closeness", sa.Float(), nullable=True),
        sa.Column("metric_label_poly_similarity", sa.Float(), nullable=True),
        sa.Column("metric_label_missing_or_broken_tracks", sa.Float(), nullable=True),
        sa.Column("metric_label_inconsistent_classification_and_track", sa.Float(), nullable=True),
        sa.Column("metric_label_shape_outlier", sa.Float(), nullable=True),
        sa.Column("metric_label_confidence", sa.Float(), nullable=False),
        sa.Column("metric_custom0", sa.Float(), nullable=True),
        sa.Column("metric_custom1", sa.Float(), nullable=True),
        sa.Column("metric_custom2", sa.Float(), nullable=True),
        sa.Column("metric_custom3", sa.Float(), nullable=True),
        sa.ForeignKeyConstraint(
            ["project_hash", "du_hash", "frame"],
            [
                "active_project_data_units.project_hash",
                "active_project_data_units.du_hash",
                "active_project_data_units.frame",
            ],
            name="active_label_project_data_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("project_hash", "du_hash", "frame", "object_hash"),
    )
    op.create_index(
        "active_label_project_hash_metric_annotation_quality_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_annotation_quality"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_area_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_area"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_area_relative_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_area_relative"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_aspect_ratio_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_aspect_ratio"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_blue_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_blue"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_brightness_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_brightness"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_contrast_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_contrast"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_green_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_green"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_height_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_height"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_label_border_closeness_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_label_border_closeness"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_label_confidence_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_label_confidence"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_label_duplicates_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_label_duplicates"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_label_inconsistent_classification_and_track_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_label_inconsistent_classification_and_track"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_label_missing_or_broken_tracks_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_label_missing_or_broken_tracks"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_label_poly_similarity_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_label_poly_similarity"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_label_shape_outlier_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_label_shape_outlier"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_random_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_random"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_red_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_red"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_sharpness_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_sharpness"],
        unique=False,
    )
    op.create_index(
        "active_label_project_hash_metric_width_index",
        "active_project_analytics_annotation",
        ["project_hash", "metric_width"],
        unique=False,
    )
    op.create_table(
        "active_project_analytics_data",
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("metric_width", sa.Integer(), nullable=True),
        sa.Column("metric_height", sa.Integer(), nullable=True),
        sa.Column("metric_area", sa.Integer(), nullable=True),
        sa.Column("metric_aspect_ratio", sa.Float(), nullable=True),
        sa.Column("metric_brightness", sa.Float(), nullable=True),
        sa.Column("metric_contrast", sa.Float(), nullable=True),
        sa.Column("metric_sharpness", sa.Float(), nullable=True),
        sa.Column("metric_red", sa.Float(), nullable=True),
        sa.Column("metric_green", sa.Float(), nullable=True),
        sa.Column("metric_blue", sa.Float(), nullable=True),
        sa.Column("metric_random", sa.Float(), nullable=True),
        sa.Column("metric_annotation_quality", sa.Float(), nullable=True),
        sa.Column("metric_object_count", sa.Integer(), nullable=True),
        sa.Column("metric_object_density", sa.Float(), nullable=True),
        sa.Column("metric_image_difficulty", sa.Float(), nullable=True),
        sa.Column("metric_image_singularity", sa.Float(), nullable=True),
        sa.Column("metric_custom0", sa.Float(), nullable=True),
        sa.Column("metric_custom1", sa.Float(), nullable=True),
        sa.Column("metric_custom2", sa.Float(), nullable=True),
        sa.Column("metric_custom3", sa.Float(), nullable=True),
        sa.ForeignKeyConstraint(
            ["project_hash", "du_hash", "frame"],
            [
                "active_project_data_units.project_hash",
                "active_project_data_units.du_hash",
                "active_project_data_units.frame",
            ],
            name="active_data_project_analytics_data_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("project_hash", "du_hash", "frame"),
    )
    op.create_index(
        "active_data_project_hash_metric_annotation_quality_index",
        "active_project_analytics_data",
        ["project_hash", "metric_annotation_quality"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_area_index",
        "active_project_analytics_data",
        ["project_hash", "metric_area"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_aspect_ratio_index",
        "active_project_analytics_data",
        ["project_hash", "metric_aspect_ratio"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_blue_index",
        "active_project_analytics_data",
        ["project_hash", "metric_blue"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_brightness_index",
        "active_project_analytics_data",
        ["project_hash", "metric_brightness"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_contrast_index",
        "active_project_analytics_data",
        ["project_hash", "metric_contrast"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_green_index",
        "active_project_analytics_data",
        ["project_hash", "metric_green"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_height_index",
        "active_project_analytics_data",
        ["project_hash", "metric_height"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_image_difficulty_index",
        "active_project_analytics_data",
        ["project_hash", "metric_image_difficulty"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_image_singularity_index",
        "active_project_analytics_data",
        ["project_hash", "metric_image_singularity"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_object_count_index",
        "active_project_analytics_data",
        ["project_hash", "metric_object_count"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_object_density_index",
        "active_project_analytics_data",
        ["project_hash", "metric_object_density"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_random_index",
        "active_project_analytics_data",
        ["project_hash", "metric_random"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_red_index",
        "active_project_analytics_data",
        ["project_hash", "metric_red"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_sharpness_index",
        "active_project_analytics_data",
        ["project_hash", "metric_sharpness"],
        unique=False,
    )
    op.create_index(
        "active_data_project_hash_metric_width_index",
        "active_project_analytics_data",
        ["project_hash", "metric_width"],
        unique=False,
    )
    op.create_table(
        "active_project_prediction_analytics_extra",
        sa.Column("prediction_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("object_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=False),
        sa.Column("annotation_bytes", sa.LargeBinary(), nullable=False),
        sa.Column("embedding_clip", sa.LargeBinary(), nullable=True),
        sa.ForeignKeyConstraint(
            ["prediction_hash", "du_hash", "frame"],
            [
                "active_project_prediction_analytics.prediction_hash",
                "active_project_prediction_analytics.du_hash",
                "active_project_prediction_analytics.frame",
            ],
            name="active_project_prediction_analytics_extra_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("prediction_hash", "du_hash", "frame", "object_hash"),
    )
    op.create_table(
        "active_project_prediction_analytics_reduced",
        sa.Column("reduction_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("prediction_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("object_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=False),
        sa.Column("x", sa.Float(), nullable=True),
        sa.Column("y", sa.Float(), nullable=True),
        sa.ForeignKeyConstraint(
            ["prediction_hash", "du_hash", "frame", "object_hash"],
            [
                "active_project_prediction_analytics.prediction_hash",
                "active_project_prediction_analytics.du_hash",
                "active_project_prediction_analytics.frame",
                "active_project_prediction_analytics.object_hash",
            ],
            name="active_data_project_prediction_analytics_reduced_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.ForeignKeyConstraint(
            ["reduction_hash"],
            ["active_project_embedding_reduction.reduction_hash"],
            name="active_data_project_prediction_analytics_reduced_reduction_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("reduction_hash", "prediction_hash", "du_hash", "frame", "object_hash"),
    )
    op.create_index(
        "active_project_analytics_prediction_reduced_x",
        "active_project_prediction_analytics_reduced",
        ["reduction_hash", "prediction_hash", "x", "y"],
        unique=False,
    )
    op.create_index(
        "active_project_analytics_prediction_reduced_y",
        "active_project_prediction_analytics_reduced",
        ["reduction_hash", "prediction_hash", "y", "x"],
        unique=False,
    )
    op.create_table(
        "active_project_analytics_annotation_extra",
        sa.Column("metric_metadata", sa.JSON(), nullable=True),
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("object_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=False),
        sa.Column("embedding_clip", sa.LargeBinary(), nullable=True),
        sa.Column("embedding_hu", sa.LargeBinary(), nullable=True),
        sa.ForeignKeyConstraint(
            ["project_hash", "du_hash", "frame"],
            [
                "active_project_analytics_annotation.project_hash",
                "active_project_analytics_annotation.du_hash",
                "active_project_analytics_annotation.frame",
            ],
            name="active_data_project_annotation_analytics_extra_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("project_hash", "du_hash", "frame", "object_hash"),
    )
    op.create_table(
        "active_project_analytics_annotation_reduced",
        sa.Column("reduction_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("object_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=False),
        sa.Column("x", sa.Float(), nullable=True),
        sa.Column("y", sa.Float(), nullable=True),
        sa.ForeignKeyConstraint(
            ["project_hash", "du_hash", "frame", "object_hash"],
            [
                "active_project_analytics_annotation.project_hash",
                "active_project_analytics_annotation.du_hash",
                "active_project_analytics_annotation.frame",
                "active_project_analytics_annotation.object_hash",
            ],
            name="active_data_project_annotation_analytics_reduced_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.ForeignKeyConstraint(
            ["reduction_hash"],
            ["active_project_embedding_reduction.reduction_hash"],
            name="active_data_project_annotation_analytics_reduced_reduction_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("reduction_hash", "project_hash", "du_hash", "frame", "object_hash"),
    )
    op.create_index(
        "active_project_analytics_annotation_reduced_x",
        "active_project_analytics_annotation_reduced",
        ["reduction_hash", "project_hash", "x", "y"],
        unique=False,
    )
    op.create_index(
        "active_project_analytics_annotation_reduced_y",
        "active_project_analytics_annotation_reduced",
        ["reduction_hash", "project_hash", "y", "x"],
        unique=False,
    )
    op.create_table(
        "active_project_analytics_data_extra",
        sa.Column("metric_metadata", sa.JSON(), nullable=True),
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("embedding_clip", sa.LargeBinary(), nullable=True),
        sa.Column("embedding_hu", sa.LargeBinary(), nullable=True),
        sa.ForeignKeyConstraint(
            ["project_hash", "du_hash", "frame"],
            [
                "active_project_analytics_data.project_hash",
                "active_project_analytics_data.du_hash",
                "active_project_analytics_data.frame",
            ],
            name="active_data_project_data_analytics_extra_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("project_hash", "du_hash", "frame"),
    )
    op.create_table(
        "active_project_analytics_data_reduced",
        sa.Column("reduction_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("x", sa.Float(), nullable=True),
        sa.Column("y", sa.Float(), nullable=True),
        sa.ForeignKeyConstraint(
            ["project_hash", "du_hash", "frame"],
            [
                "active_project_analytics_data.project_hash",
                "active_project_analytics_data.du_hash",
                "active_project_analytics_data.frame",
            ],
            name="active_data_project_data_analytics_reduced_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.ForeignKeyConstraint(
            ["reduction_hash"],
            ["active_project_embedding_reduction.reduction_hash"],
            name="active_data_project_data_analytics_reduced_reduction_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("reduction_hash", "project_hash", "du_hash", "frame"),
    )
    op.create_index(
        "active_project_analytics_data_reduced_x",
        "active_project_analytics_data_reduced",
        ["reduction_hash", "project_hash", "x", "y"],
        unique=False,
    )
    op.create_index(
        "active_project_analytics_data_reduced_y",
        "active_project_analytics_data_reduced",
        ["reduction_hash", "project_hash", "y", "x"],
        unique=False,
    )
    op.create_table(
        "active_project_tagged_annotation",
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("object_hash", sqlmodel.sql.sqltypes.AutoString(length=8), nullable=False),
        sa.Column("tag_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.ForeignKeyConstraint(
            ["project_hash", "du_hash", "frame", "object_hash"],
            [
                "active_project_analytics_annotation.project_hash",
                "active_project_analytics_annotation.du_hash",
                "active_project_analytics_annotation.frame",
                "active_project_analytics_annotation.object_hash",
            ],
            name="active_project_tagged_objects_analysis_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.ForeignKeyConstraint(
            ["tag_hash"],
            ["active_project_tags.tag_hash"],
            name="active_project_tagged_objects_tag_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("project_hash", "du_hash", "frame", "object_hash", "tag_hash"),
    )
    op.create_index(
        op.f("ix_active_project_tagged_annotation_tag_hash"),
        "active_project_tagged_annotation",
        ["tag_hash"],
        unique=False,
    )
    op.create_table(
        "active_project_tagged_data",
        sa.Column("project_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("du_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.Column("frame", sa.Integer(), nullable=False),
        sa.Column("tag_hash", sqlmodel.sql.sqltypes.GUID(), nullable=False),
        sa.ForeignKeyConstraint(
            ["project_hash", "du_hash", "frame"],
            [
                "active_project_analytics_data.project_hash",
                "active_project_analytics_data.du_hash",
                "active_project_analytics_data.frame",
            ],
            name="active_project_tagged_data_units_analysis_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.ForeignKeyConstraint(
            ["tag_hash"],
            ["active_project_tags.tag_hash"],
            name="active_project_tagged_data_units_tag_fk",
            onupdate="CASCADE",
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("project_hash", "du_hash", "frame", "tag_hash"),
    )
    op.create_index(
        op.f("ix_active_project_tagged_data_tag_hash"), "active_project_tagged_data", ["tag_hash"], unique=False
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_active_project_tagged_data_tag_hash"), table_name="active_project_tagged_data")
    op.drop_table("active_project_tagged_data")
    op.drop_index(op.f("ix_active_project_tagged_annotation_tag_hash"), table_name="active_project_tagged_annotation")
    op.drop_table("active_project_tagged_annotation")
    op.drop_index("active_project_analytics_data_reduced_y", table_name="active_project_analytics_data_reduced")
    op.drop_index("active_project_analytics_data_reduced_x", table_name="active_project_analytics_data_reduced")
    op.drop_table("active_project_analytics_data_reduced")
    op.drop_table("active_project_analytics_data_extra")
    op.drop_index(
        "active_project_analytics_annotation_reduced_y", table_name="active_project_analytics_annotation_reduced"
    )
    op.drop_index(
        "active_project_analytics_annotation_reduced_x", table_name="active_project_analytics_annotation_reduced"
    )
    op.drop_table("active_project_analytics_annotation_reduced")
    op.drop_table("active_project_analytics_annotation_extra")
    op.drop_index(
        "active_project_analytics_prediction_reduced_y", table_name="active_project_prediction_analytics_reduced"
    )
    op.drop_index(
        "active_project_analytics_prediction_reduced_x", table_name="active_project_prediction_analytics_reduced"
    )
    op.drop_table("active_project_prediction_analytics_reduced")
    op.drop_table("active_project_prediction_analytics_extra")
    op.drop_index("active_data_project_hash_metric_width_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_sharpness_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_red_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_random_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_object_density_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_object_count_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_image_singularity_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_image_difficulty_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_height_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_green_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_contrast_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_brightness_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_blue_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_aspect_ratio_index", table_name="active_project_analytics_data")
    op.drop_index("active_data_project_hash_metric_area_index", table_name="active_project_analytics_data")
    op.drop_index(
        "active_data_project_hash_metric_annotation_quality_index", table_name="active_project_analytics_data"
    )
    op.drop_table("active_project_analytics_data")
    op.drop_index("active_label_project_hash_metric_width_index", table_name="active_project_analytics_annotation")
    op.drop_index("active_label_project_hash_metric_sharpness_index", table_name="active_project_analytics_annotation")
    op.drop_index("active_label_project_hash_metric_red_index", table_name="active_project_analytics_annotation")
    op.drop_index("active_label_project_hash_metric_random_index", table_name="active_project_analytics_annotation")
    op.drop_index(
        "active_label_project_hash_metric_label_shape_outlier_index", table_name="active_project_analytics_annotation"
    )
    op.drop_index(
        "active_label_project_hash_metric_label_poly_similarity_index", table_name="active_project_analytics_annotation"
    )
    op.drop_index(
        "active_label_project_hash_metric_label_missing_or_broken_tracks_index",
        table_name="active_project_analytics_annotation",
    )
    op.drop_index(
        "active_label_project_hash_metric_label_inconsistent_classification_and_track_index",
        table_name="active_project_analytics_annotation",
    )
    op.drop_index(
        "active_label_project_hash_metric_label_duplicates_index", table_name="active_project_analytics_annotation"
    )
    op.drop_index(
        "active_label_project_hash_metric_label_confidence_index", table_name="active_project_analytics_annotation"
    )
    op.drop_index(
        "active_label_project_hash_metric_label_border_closeness_index",
        table_name="active_project_analytics_annotation",
    )
    op.drop_index("active_label_project_hash_metric_height_index", table_name="active_project_analytics_annotation")
    op.drop_index("active_label_project_hash_metric_green_index", table_name="active_project_analytics_annotation")
    op.drop_index("active_label_project_hash_metric_contrast_index", table_name="active_project_analytics_annotation")
    op.drop_index("active_label_project_hash_metric_brightness_index", table_name="active_project_analytics_annotation")
    op.drop_index("active_label_project_hash_metric_blue_index", table_name="active_project_analytics_annotation")
    op.drop_index(
        "active_label_project_hash_metric_aspect_ratio_index", table_name="active_project_analytics_annotation"
    )
    op.drop_index(
        "active_label_project_hash_metric_area_relative_index", table_name="active_project_analytics_annotation"
    )
    op.drop_index("active_label_project_hash_metric_area_index", table_name="active_project_analytics_annotation")
    op.drop_index(
        "active_label_project_hash_metric_annotation_quality_index", table_name="active_project_analytics_annotation"
    )
    op.drop_table("active_project_analytics_annotation")
    op.drop_index(
        "active_project_prediction_unmatched_feature_hash_index",
        table_name="active_project_prediction_analytics_false_negatives",
    )
    op.drop_table("active_project_prediction_analytics_false_negatives")
    op.drop_index(
        "active_project_prediction_objects_feature_confidence_index", table_name="active_project_prediction_analytics"
    )
    op.drop_index(
        "active_project_prediction_objects_confidence_index", table_name="active_project_prediction_analytics"
    )
    op.drop_table("active_project_prediction_analytics")
    op.drop_index("active_project_data_units_unique_du_hash_frame", table_name="active_project_data_units")
    op.drop_table("active_project_data_units")
    op.drop_index(op.f("ix_active_project_tags_project_hash"), table_name="active_project_tags")
    op.drop_table("active_project_tags")
    op.drop_table("active_project_prediction")
    op.drop_table("active_project_embedding_reduction")
    op.drop_table("active_project_data")
    op.drop_table("active_project")
    # ### end Alembic commands ###
